<?php
/*
Plugin Name: Grouped Links Widget
Description: Grouped Links Widget can use any number of widget instances to group links by their associated link categories. Tested for WP 2.6 only.
Author: Stefan Meretz
Version: 1.0
Author URI: http://www.meretz.de
Plugin URI: http://groupedlinks.wordpress.com/

Copyright (C) 2008 Stefan Meretz

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

// max number of grouped links widgets -- increase, if necessary
$max_links_groups = 8;
///////////////////////////////////////////////////////////////////

// generate the output for the sidebars
function grouped_links_widget( $args, $number ) {
	echo $args['before_widget'];
	$opt = get_option( 'widget_grouped_links' );
	$categories = $opt[$number]['category_group'];
	$cat_order = $opt[$number]['category_order'];
	// to use wp_list_bookmarks see http://codex.wordpress.org/wp_list_bookmarks
	wp_list_bookmarks( array( 'category' => $categories, 'category_order' => $cat_order ));
	echo $args['after_widget'];
}

// configuring up to 9 instances of the widget
function grouped_links_widget_control( $number ) {
	global $wpdb;
	$opt = $newopt = get_option( 'widget_grouped_links' );
	$gid = '';
	if( $_POST["grouped_links-submit-$number"] ) {
		for( $i = 1; $i <= $_POST["grouped_links-submit-$number"]; $i++ ) {
			if( $_POST["grouped_links-category_group-$number-$i"] ) {
				if( !empty( $gid )) { $gid .= ','; }
				$gid .= $_POST["grouped_links-category_group-$number-$i"];
			}
		}
		$newopt[$number]['category_group'] = $gid;
		$newopt[$number]['category_order'] = $_POST["grouped_links-category_order-$number"];
	}
	if( $opt != $newopt ) {
		$opt = $newopt;
		update_option( 'widget_grouped_links', $opt );
	}
	$cat_order = $opt[$number]['category_order'];
	$gid = strtok( $opt[$number]['category_group'], "," );
	while( $gid !== false ) {
		$gid_list[$gid] = 1;
		$gid = strtok( "," );
	}
	$qry = "SELECT t.term_id id, t.name cat FROM $wpdb->terms t, $wpdb->term_taxonomy x ";
	$qry .= "WHERE t.term_id = x.term_id and x.taxonomy = 'link_category' and x.count > 0 order by cat $cat_order";
	$results = $wpdb->get_results( $qry );
	echo "\t\t\t<p>\n\t\t\t";
	_e( 'Link Categories' );
	echo ":\n\t\t\t</p>\n\t\t\t<p>\n";
	if( $results ) {
		$i = 1;
		foreach( $results as $result ) {
			$cat_gid = $result->id;
			$cat_group = $result->cat;
			echo "\t\t\t<label>\n\t\t\t<input class='checkbox' type='checkbox' ";
			echo "name='grouped_links-category_group-$number-$i' value='$cat_gid'";
			echo $gid_list[$cat_gid] ? ' checked' : '';
			echo " /> $cat_group<br />\n\t\t\t</label>\n";
			$i++;
		}
	}
	echo "\t\t\t</p>\n\t\t\t<p>\n";
	echo "\t\t\t<label>Sort order:\n\t\t\t<select name='grouped_links-category_order-$number'>\n";
	echo "\t\t\t\t<option value='ASC'".( $cat_order == 'ASC' ? " selected='selected'" : '' ).">acending</option>\n";
	echo "\t\t\t\t<option value='DESC'".( $cat_order == 'DESC' ? " selected='selected'" : '' ).">decending</option>\n";
	echo "\t\t\t</select>\n\t\t\t</label>\n";
	echo "\t\t\t</p>\n\t\t\t<input type='hidden' name='grouped_links-submit-$number' value='$i' />\n";
}

// register and unregister sidebar output and widgets control functions
function grouped_links_widget_register() {
	$opt = get_option( 'widget_grouped_links' );
	$number = $opt['number'];
	for( $i = 1; $i <= 9; $i++ ) {
		$name = array( 'Grouped Links %s', null, $i );
		if( $i <= $number ) {
			register_sidebar_widget( $name, 'grouped_links_widget', $i );
			register_widget_control( $name, 'grouped_links_widget_control', '', '', $i );
		} else {
			unregister_sidebar_widget( $name );
			unregister_widget_control( $name );
		}
	}
}

// hook in sidebar output and widgets control un-/register function
add_action( 'plugins_loaded', 'grouped_links_widget_register' );
///////////////////////////////////////////////////////////////////

// options page to set number of plugin instances
function grouped_links_widget_options() {
	global $max_links_groups;
	$opt = get_option( 'widget_grouped_links' );
	if( isset( $_POST['grouped_links-number-submit'] )) {
		$opt['number'] = $_POST['grouped_links-number'];
		update_option( 'widget_grouped_links', $opt );
		echo "<div class='updated'>\n\t<p><strong>";
		_e( 'Options saved.' );
		echo "</strong></p>\n</div>\n";
	}
	echo "<div class='wrap'>\n";
	echo "\t<form method='post'>\n";
	echo "\t\t<h2>Grouped Links</h2>\n";
	echo "\t\t<p>\n\t\t";
	_e( 'Number of Link-Groups: ' );
	echo "\n\t\t<select name='grouped_links-number' value='".$opt['number']."'>";
	for( $i = 1; $i <= $max_links_groups; ++$i ) {
		echo "\n\t\t\t<option value='$i' ".( $opt['number'] == $i ? "selected='selected'" : '' ).">$i</option>";
	}
	echo "\n\t\t</select>\n";
	echo "\t\t<span class='submit'>\n";
	echo "\t\t\t<input type='submit' name='grouped_links-number-submit' value='";
	_e( 'Save' );
	echo "' />\n\t\t</span>\n\t\t</p>\n\t</form>\n</div>\n";
}

// register options page to set number of plugin instances
function grouped_links_widget_options_register() {
	add_management_page( 'Grouped Links Options', 'Grouped Links', 8, __FILE__, 'grouped_links_widget_options' );
}

// hook in options register function
add_action( 'admin_menu', 'grouped_links_widget_options_register');
///////////////////////////////////////////////////////////////////

// initialize grouped links using one widget instance
function grouped_links_widget_init() {
	$opt['number'] = 1;
	$opt[1]['category_group'] = '';
	add_option( 'widget_grouped_links', $opt );
}

// delete grouped links options from database
function grouped_links_widget_delete() {
	delete_option( 'widget_grouped_links' );
}

// hook in init and delete functions
add_action( 'init', 'grouped_links_widget_init' );
add_action( 'deactivate_'.plugin_basename(__FILE__), 'grouped_links_widget_delete' );
///////////////////////////////////////////////////////////////////

?>
